<?php declare(strict_types=1);

namespace App\Http\Controllers\Api\V1;

use App\Http\Requests\Api\V1\Captcha\CaptchaRequest;
use App\Http\Resources\Api\V1\Captcha;
use App\Services\Api\V1\CaptchaService;
use Illuminate\Http\JsonResponse;

final class CaptchaController extends Controller
{
    public function __construct(
        private readonly CaptchaService $captchaService
    ) { }

    public function getCaptcha(CaptchaRequest $request): JsonResponse
    {
        $captchaPublicToken = $request->getDto();
        $expires = now()->addSeconds(config('captcha.waiting_for_captcha_verification_in_seconds'));
        $result = $this->captchaService->createKeyWithCaptcha($captchaPublicToken, $expires);
        if (!$result->isSuccess()) {
            return response()->json($result->getData())->setStatusCode($result->getCode());
        }

        return response()->json(new Captcha($result));
    }
}
